`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/04/02 16:30:11
// Design Name: 
// Module Name: topmodule
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

`define ENABLE 1'b1
module topmodule(
    D_ssd,
    ssd_ctl,
    clk,
    rst_n,
    select_mode,
    select_clk
    );

    output [7:0] D_ssd;
    output [3:0] ssd_ctl;
    input clk;
    input rst_n;
    input select_mode;
    input select_clk;
    
    wire clk_1Hz;
    wire clk_100Hz;
    wire [1:0] clk_ssd_ctl;

    wire debounced_in, debounced_in_clk;
    wire out_pulse, out_pulse_clk;

    wire mode_sel_result;
    wire curr_state_clk;
    wire clk_out_for_count;
    
    wire [3:0] digit_3, digit_2, digit_1, digit_0;
    wire [3:0] digit_1_mon, digit_0_mon;
    wire [3:0] digit_1_day, digit_0_day;
    wire [3:0] digit_1_year, digit_0_year;

    wire [3:0] ssd_in_month, ssd_in_year;
    wire [3:0] ssd_ctl_month, ssd_ctl_year;

    wire [7:0] D_ssd_month, D_ssd_tmp, D_ssd_year;

    wire [3:0] ssd_in;
    //wire [3:0] D_ssd_tmp, ssd_in_tmp;
   

    clock_generator U0(
    .clk_1Hz(clk_1Hz), //divided clock output
    .clk_100Hz(clk_100Hz),
    .clk_ctl(clk_ssd_ctl), //clock for 7-segment
    .clk(clk),    //clock input
    .rst_n(rst_n)   //low-active reset
    );

    /*debounce U1( 
    .rst(rst_n), 
    .clk(clk_100Hz), 
    .pb_in(select_mode),
    .pb_debounced(debounced_in)
    );*/

    debounce U11( 
    .rst(rst_n), 
    .clk(clk_100Hz), 
    .pb_in(select_clk),
    .pb_debounced(debounced_in_clk)
    );

    /*one_pulse U2( 
    .rst_n(rst_n), 
    .clk(clk_100Hz), 
    .in_trig(debounced_in),
    .out_pulse(out_pulse)
);*/
    one_pulse U22( 
    .rst_n(rst_n), 
    .clk(clk_100Hz), 
    .in_trig(debounced_in_clk),
    .out_pulse(out_pulse_clk)
);
    
    /*fsm U3(
    .select_mode_change(out_pulse),
    .clk(clk_100Hz),
    .rst_n(rst_n),
    .current_state(mode_sel_result)
);*/
    fsm U33(
    .select_mode_change(out_pulse_clk),
    .clk(clk_100Hz),
    .rst_n(rst_n),
    .current_state(curr_state_clk)
    );

    select_clk U333(
    .clk_1Hz(clk_1Hz),
    .clk_100Hz(clk_100Hz),
    .clk_out_for_count(clk_out_for_count),
    .select(curr_state_clk)
);

    BCD_up_month_date U4(
    .digit_3(digit_1_mon),
    .digit_2(digit_0_mon),
    .digit_1(digit_1_day),
    .digit_0(digit_0_day),
    .rst_n(rst_n),
    .clk(clk_out_for_count),
    .en(`ENABLE),
    .cout(enable_year)
    );

    BCD_up_year U5(
    .digit_1(digit_1_year),
    .digit_0(digit_0_year),
    .rst_n(rst_n),
    .clk(clk_out_for_count),
    .en(enable_year)
    );
   
    scan_freq_ctl U6(
    .ssd_in(ssd_in_month),
    .ssd_ctl(ssd_ctl_month),
    .in_0(digit_0_day),
    .in_1(digit_1_day),
    .in_2(digit_0_mon),
    .in_3(digit_1_mon),
    .clk_ctl_en(clk_ssd_ctl),
    .rst_n(rst_n)
    );

    scan_freq_ctl U61(
    .ssd_in(ssd_in_year),
    .ssd_ctl(ssd_ctl_year),
    .in_0(digit_0_year),
    .in_1(digit_1_year),
    .in_2(4'd0),
    .in_3(4'd0),
    .clk_ctl_en(clk_ssd_ctl),
    .rst_n(rst_n)
    );

    ssd_selector U7(
    .ssd_ctl(ssd_ctl),
    .ssd_ctl_month(ssd_ctl_month),
    .ssd_ctl_year(ssd_ctl_year),
    .ssd_in(ssd_in),
    .ssd_in_month(ssd_in_month),
    .ssd_in_year(ssd_in_year),
    .select_mode(select_mode)
);

    display U8(.D_ssd(D_ssd), .i(ssd_in));

    
endmodule
